%%%%%%%%%
% TAR.M %
%%%%%%%%%

% Bruce E. Hansen
% Department of Economics
% Social Science Building
% University of Wisconsin
% Madison, WI 53706-1393
% bhansen@ssc.wisc.edu
% http://www.ssc.wisc.edu/~bhansen/



% This program estimates a p'th order threshold autoregression and tests the
% hypothesis of a linear autoregression, using the statistics described in
% "Inference when a nuisance parameter is not identified under the null
% hypothesis."

% The procedure is a function of eight inputs, and takes the form

%  [tests,pvalues] = tar(y,p,omit,r1,r2,rep,out,graph);

% The inputs are:

% y     =  the vector of observations
% p     =  the order of the autoregression
% omit  =  lags (below p) to omit from autoregression [0 implies an AR(p)]
% r1    =  the lower quantile
% r2    =  the upper quantile
% rep   =  number of simulation replications
% out   =  indicator variable for screen output
% graph =  indicator variable for graphical output


% Additional detail on inputs:

% p       should be a positive integer

% omit    if 0, then an AR(p) model is used
%         if a positive integer, then that AR lag is omitted from model
%         if a vector of positive integers, then all these lags are omitted

% out     should be either 0 or 1
%         if 0, then nothing is printed to the screen nor to an output file
%         if 1, then OLS estimates and test statistics are printed to the
%         screen and to an output file (if previously declared)

% graph   should be either 0 or 1
%         if 0, then no graphs are generated
%         if 1, then graphs of the test statistic sequences are displayed

% There are two output vectors, the first contains the test statistics, and the
% second the estimated asymptotic p-values.  The statistics (in order) are:
% SupLM, ExpLM, AveLM, SupLMs, ExpLMs, AveLMs.



%Construct Data%
function [tests,pvalues]=tar(dat,p,omit,r1,r2,rep,ot,gr)

t=length(dat)-p;

if omit==0
    k=p;
else
    ro=length(omit);
    k=p-ro;
end;

k1=k+1;
tk1=t-k1;
tk2=t-k1*2;
y=dat(1+p:t+p);
x=ones(t,1);

j=1;
while j<=p
    sumc=0;
    for i=1:length(omit)
        if omit(i)==j
            sumc=sumc+1;
        end;
    end;
    if sumc==0
        x=[x,dat(1+p-j:t+p-j)];
    end;
    j=j+1;
end;

%Null Model%
s=x'*y;
m=x'*x;
mi=inv(m);
bnull=mi*s;
u=y-x*bnull;
sig=(u'*u)/tk1;



%Standard Error of Null Model%
kx=sum((x').*(mi*(x')))';

xu=x(:,1).*(u./(1-kx));
i=2;
while i<=length(x(1,:))
    xu=[xu,x(:,i).*(u./(1-kx))];
    i=i+1;
end;

xu=xu-ones(length(xu(:,1)),1)*(mean(xu));
v=xu'*xu;
se=sqrt(diag(mi*v*mi));

%Output Print%
avr=0; %The Index of Uesd variables%
if ot==1
    disp('OLS Estimation of Null Linear Model');
    disp('-----------------------------------');
    disp('Variable    Estimate      S.E.' );
    fprintf('Intercept   %f      %f\n',bnull(1),se(1));
    j=1;
    for i=1:p
        if sum(omit==i)==0
            fprintf('yt-%u        %f      %f\n',i,bnull(j+1),se(j+1));
            if j==1
                avr=i;
            else
                avr=[avr,i];
            end;
            j=j+1;            
        end;
    end;
    fprintf('\nResidual Variance  %f\n',sig); 
     disp(' '); disp(' '); disp(' ');
end;

%Test Statistic Storage%
t1=round(t*r1);
t2=round(t*r2);
tn=t2-t1;

lm=zeros(tn,k); 
lms=lm;  
sigr=lm;
lmmax=zeros(rep,1); 
lmsum=lmmax; 
lmsume=lmmax;
smax=lmmax; 
ssum=lmmax; 
ssume=lmmax;

sim=randn(t,rep);
xu=[x(:,1).*u];
i=2;
while i<=length(x(1,:))
    xu=[xu,x(:,i).*u];
    i=i+1;
end;
xmi=x*mi;
misim=xmi'*sim;
missim=mi*(xu'*sim);

d=1;
while d<=k
    
    if ot==1
        %"Searching over Threshold Variable:";%
    end;
    
                             
    z=0;
     %rankindx%
    for i=1:t
        index=0;
        for j=1:t
            if ((x(j,d+1)<=x(i,d+1)&(i>=j)))|(x(j,d+1)<x(i,d+1))
                index=index+1;
            end;
        end;
        if i==1
            z=index;
        end;
        if i>1
            z=[z;index];
        end;
    end;
    
     %Get zr%
    zr=0;
    for i=1:t
        index=0;
        for j=1:t
            if z(j)==i
                index=j;
            end;
        end;
        if i==1
            zr=index;
        end;
        if i>1
            zr=[zr;index];
        end;
    end;
        
    xr=[x(:,1).*(z<=t1)];
    i=2;
    while i<=length(x(1,:))
        xr=[xr,x(:,i).*(z<=t1)];
        i=i+1;
    end; 
    
    xz1=zr(1:t1);
    x1=x(xz1,:);
    mr=x1'*x1;
    sim1=sim(xz1,:);
    
    xrsim=x1'*sim1;
    xusim=xu(xz1,:)'*sim1;
    
    r=1;
    while r<=tn;  %Pick Threshold%
        
        t1r=t1+r;
        xz=zr(1:t1r);
        xz2=zr(t1r+1:t);
        zrt=zr(t1r);
        xsr=x(zrt,:);
        xr(zrt,1:k1)=xsr;
        mr=mr+xsr'*xsr;
        xt=xr-xmi*mr;
        mrir=mr*mi*mr;
        xti=inv(mr-mrir);
        xtu=xr'*u;
        br=xti*xtu;
        ur=u-xt*br;
        sigr(r,d)=ur'*ur;
        
        xru1m=xt(:,1).*u;
        i=2;
        while i<=length(xt(1,:))
            xru1m=[xru1m,xt(:,i).*u];
            i=i+1;
        end;
        
        vr1mt=inv(xru1m'*xru1m);
        lm(r,d)=xtu'*vr1mt*xtu;
        lms(r,d)=(xtu'*xti*xtu)/sig;
        
        %Simulation Manipulations%
        
        xrsim=xrsim+xsr'*sim(zrt,:);
        sims=xrsim-mr*misim;
        
        wrep=(sum((sims.*(xti*sims))))';
        smax=(max([smax';wrep']))';
        ssum=ssum+wrep;     
        ssume=ssume+exp(wrep/2);
         
        xusim=xusim+xu(zrt,:)'*sim(zrt,:);
        sims=xusim-mr*missim;
        wrep=(sum((sims.*(vr1mt*sims))))';
        lmmax=(max([lmmax';wrep']))';
        lmsum=lmsum+wrep;
        lmsume=lmsume+exp(wrep/2);
        
        
        
        
        r=r+1;
    end;
    d=d+1;
end;

clear sim;clear misim;clear missim;

%Test Statistics%
tests=[max(max(lm)');log(mean(mean(exp(lm/2))'));mean(mean(lm)');max(max(lms)');log(mean(mean(exp(lms/2))'));mean(mean(lms)')];

%P-Values%

tnk=tn*k;
ssum=ssum/tnk;
lmsum=lmsum/tnk;
ssume=log(ssume/tnk);
lmsume=log(lmsume/tnk);

pvalues=mean([lmmax,lmsume,lmsum,smax,ssume,ssum]>(ones(length(lmmax),1)*tests'))';
clear lmmax;clear lmsume;clear lmsum;clear smax;clear ssume;clear ssum;

if ot==1
    %Findout The Best d and r%
    rm=1;dm=1;
    for i=1:tn
        for j=1:k
            if sigr(rm,dm)>sigr(i,j)
                rm=i;
                dm=j;
            end;
        end;
    end;
    yx=sortrows([y,x],dm+2);
    ys=yx(:,1);
    xs=yx(:,2:k+2);
    xr=[[xs(1:t1+rm,:);zeros(t-t1-rm,k1)],[zeros(t1+rm,k1);xs(t1+rm+1:t,:)]];
    dri=inv(xr'*xr);
    br=dri*(xr'*ys);
    ur=ys-xr*br;
    kx=sum((xr'.*(dri*xr')))';
    xu=xr(:,1).*(ur./(1-kx));
    i=2;
    while i<=length(xr(1,:))
        xu=[xu,xr(:,i).*(ur./(1-kx))];
        i=i+1;
    end;
    vrw=(xu-ones(length(xu(:,1)),1)*(mean(xu)))'*(xu-ones(length(xu(:,1)),1)*(mean(xu)));
    se=sqrt(diag(dri*vrw*dri));
    sig=(ur'*ur)/(t-2*k1);
    sig1=(ur(1:t1+rm)'*ur(1:t1+rm))/(t1+rm-k1);
    sig2=(ur(t1+rm+1:t)'*ur(t1+rm+1:t))/(t-t1-rm-k1);

    %Output%
   
    disp('Global Estimate');
    disp('----------------');
    th=yx(t1+rm,dm+2);
    fprintf('Threshold Variable Lag: %uth\n', dm);
    fprintf('Threshold Estimate: %f\n',th);
    fprintf('Error Variance:%f\n\n',sig);
    disp(' ');
    fprintf('Regime 1 (yt-%u)<%f\n',dm,th);
    disp('---------------------');
    disp('Variable    Estimate      S.E.' );
    fprintf('Intercept   %f      %f\n',br(1),se(1));
    for i=1:length(avr)
        fprintf('yt-%u        %f      %f\n',avr(i),br(i+1),se(i+1));
    end;
    fprintf('Regime 1 Error Variance %f\n',sig1);
    
    disp(' ');
    fprintf('Regime 2 (yt-%u)>%f\n',dm,th);
    disp('---------------------');
    disp('Variable    Estimate      S.E.' );
    fprintf('Intercept   %f      %f\n',br(2+k),se(2+k));
    for i=1:length(avr)
        fprintf('yt-%u        %f      %f\n',avr(i),br(i+2+k),se(i+2+k));
    end;
    fprintf('Regime 2 Error Variance %f\n',sig2);
    disp(' ');
    disp('Robust LM Statistics');
     disp('---------------------');
    fprintf('SupLM    %f     %f\n',tests(1),pvalues(1));
    fprintf('ExpLM    %f     %f\n',tests(2),pvalues(2));
    fprintf('AveLM    %f     %f\n',tests(3),pvalues(3));
    disp(' ');
    disp('Standard LM Statistics');
    disp('---------------------');
    fprintf('SupLMs   %f     %f\n',tests(4),pvalues(4));
    fprintf('ExpLMs   %f     %f\n',tests(5),pvalues(5));
    fprintf('AveLMs   %f     %f\n',tests(6),pvalues(6));
 end;
 
 %Pictures%
 if gr==1
     for d=1:length(avr)
         z=sortrows(yx(:,d+2),1);
         zt=z(t1+1:t2);
         figure;
         plot(zt,lms(:,d),zt,lm(:,d)), title(strcat('Figure',num2str(d))),xlabel(strcat('yt-',num2str(avr(d)))),ylabel('Pointwise LM Statistics'),legend('Standard','Hetero-Robust'),grid on;
     end;
 end;
     


 
 %LATEX%
 if ot==1
    ttex=fopen('out.tex','a+');
    fprintf(ttex,'\\underline{\\large{Global Estimate}}\\\\\n');
    th=yx(t1+rm,dm+2);
    fprintf(ttex,'\\textbf{Threshold Variable Lag:}%uth\\\\\n', dm);
    fprintf(ttex,'\\textbf{Threshold Estimate:} %f\\\\\n',th);
    fprintf(ttex,'\\textbf{Error Variance:}%f\\\\\n',sig);
    fprintf(ttex,'\\vspace{.2cm}\\\\\n');
    
    fprintf(ttex,'\\underline{Regime 1: $y_{t-%u}\\leq%f$}\\\\\n',dm,th);
    fprintf(ttex,'\\begin{tabular}{|l |l |l |}\n');
    fprintf(ttex,'\\hline\n');
    fprintf(ttex,'Variable & Estimate & St Error\\\\\n');
    fprintf(ttex,'\\hline\\hline\n');
    fprintf(ttex,'Intercept&%f&%f\\\\\n',br(1),se(1));
    for i=1:length(avr)
        fprintf(ttex,'$y_{t-%u}$',avr(i));
        fprintf(ttex,'& %f & %f\\\\\n',br(i+1),se(i+1));
    end;
    fprintf(ttex,'\\hline\n');
    fprintf(ttex,'\\end{tabular}\\\\\n');
    fprintf(ttex,'\\textbf{Regime 1 Error Variance:}%f\\\\\n',sig1);
    fprintf(ttex,'\\vspace{.2cm}\\\\\n');
    
    fprintf(ttex,'\\underline{Regime 2: $y_{t-%u}\\leq%f$}\\\\\n',dm,th);
    fprintf(ttex,'\\begin{tabular}{|l |l |l |}\n');
    fprintf(ttex,'\\hline\n');
    fprintf(ttex,'Variable & Estimate & St Error\\\\\n');
    fprintf(ttex,'\\hline\\hline\n');
    fprintf(ttex,'Intercept&%f&%f\\\\\n',br(2+k),se(2+k));
    for i=1:length(avr)
        fprintf(ttex,'$y_{t-%u}$',avr(i));
        fprintf(ttex,'& %f & %f\\\\\n',br(i+2+k),se(i+2+k));
    end;
    fprintf(ttex,'\\hline\n');
    fprintf(ttex,'\\end{tabular}\\\\\n');
    fprintf(ttex,'\\textbf{Regime 2 Error Variance:}%f\\\\\n',sig2);
    fprintf(ttex,'\\vspace{.2cm}\\\\\n');
    
    
    fprintf(ttex,'\\underline{\\large{Robust LM Statistics}}\\\\\n');
    fprintf(ttex,'\\begin{tabular}{l l l }\n');
    fprintf(ttex,'\\textbf{SupLM:}&%f&%f\\\\\n',tests(1),pvalues(1));
    fprintf(ttex,'\\textbf{ExpLM:}&%f&%f\\\\\n',tests(2),pvalues(2));
    fprintf(ttex,'\\textbf{AveLM:}&%f&%f\\\\\n',tests(3),pvalues(3));
    fprintf(ttex,'\\end{tabular}\n');
    fprintf(ttex,'\\vspace{.1cm}\\\\\n');
    
    fprintf(ttex,'\\underline{\\large{Standard LM Statistics}}\\\\\n');
    fprintf(ttex,'\\begin{tabular}{l l l }\n');
    fprintf(ttex,'\\textbf{SupLM:}&%f&%f\\\\\n',tests(4),pvalues(4));
    fprintf(ttex,'\\textbf{ExpLM:}&%f&%f\\\\\n',tests(5),pvalues(5));
    fprintf(ttex,'\\textbf{AveLM:}&%f&%f\\\\\n',tests(6),pvalues(6));
    fprintf(ttex,'\\end{tabular}\n');
    fprintf(ttex,'\\vspace{.2cm}\\\\\n');
    fclose(ttex);
 end;
        



